/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.admin.ui.action;

import org.apache.struts2.StrutsStatics;
import org.wso2.solutions.identity.admin.RealmConfigAdmin;
import org.wso2.solutions.identity.admin.ui.UIConstants;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionSupport;

public class CreateRealmConfigurationAction extends ActionSupport {

    private static final long serialVersionUID = -5877313929595725808L;

    private ArrayList missingProperties;

    private HashMap propertyMap;

    public String execute() throws Exception {

        HttpServletRequest request = (HttpServletRequest) ActionContext
                .getContext().get(StrutsStatics.HTTP_REQUEST);
        HttpSession session = request.getSession();
        String realmClassName = (String) session
                .getAttribute(UIConstants.REALM_CLASS_NAME);
        List configProperties = (List) session
                .getAttribute(UIConstants.REALM_CONFIG_PROPERTIES);

        // If something wrong with the session
        if (realmClassName == null || configProperties == null) {
            return ERROR;
        }

        propertyMap = new HashMap();
        missingProperties = new ArrayList();
        for (Iterator iterator = configProperties.iterator(); iterator
                .hasNext();) {
            String prop = (String) iterator.next();
            String propValue = request.getParameter(prop);
            if (propValue == null || "".equals(propValue.trim())) {
                // The property is missing
                missingProperties.add(prop);
            } else {
                propertyMap.put(prop, propValue);
            }
        }

        String configName = request
                .getParameter(UIConstants.NEW_REALM_CONFIG_NAME);
        if (configName == null) {
            missingProperties.add(UIConstants.NEW_REALM_CONFIG_NAME);
        }

        // If we have any missing properties then show error page
        if (missingProperties.size() > 0) {
            // TODO Add code to the action class and the JSP for validation
            return "validationError";
        }

        RealmConfigAdmin admin = new RealmConfigAdmin();
        admin.createNewRealmConfiguration(realmClassName, configName,
                propertyMap);

        return SUCCESS;
    }

    public ArrayList getMissingProperties() {
        return missingProperties;
    }

    public HashMap getPropertyMap() {
        return propertyMap;
    }

}
