/*                                                                             
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *                                                                             
 * Licensed under the Apache License, Version 2.0 (the "License");             
 * you may not use this file except in compliance with the License.            
 * You may obtain a copy of the License at                                     
 *                                                                             
 *      http://www.apache.org/licenses/LICENSE-2.0                             
 *                                                                             
 * Unless required by applicable law or agreed to in writing, software         
 * distributed under the License is distributed on an "AS IS" BASIS,           
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    
 * See the License for the specific language governing permissions and         
 * limitations under the License.                                              
 */
package org.wso2.solutions.identity;

import java.util.ArrayList;
import java.util.Iterator;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.security.handler.WSHandlerConstants;
import org.wso2.solutions.identity.admin.ClaimsAdmin;
import org.wso2.solutions.identity.admin.ConfigurationContextHolder;
import org.wso2.solutions.identity.admin.STSConfigAdmin;
import org.wso2.solutions.identity.i18n.Messages;
import org.wso2.solutions.identity.persistence.IPPersistenceManager;
import org.wso2.solutions.identity.persistence.dataobject.ActionDO;
import org.wso2.solutions.identity.persistence.dataobject.ClaimDO;
import org.wso2.solutions.identity.persistence.dataobject.DialectDO;
import org.wso2.solutions.identity.persistence.dataobject.RealmConfigurationDO;
import org.wso2.solutions.identity.persistence.dataobject.RealmConfigurationPropertyDO;
import org.wso2.solutions.identity.persistence.dataobject.RealmDO;
import org.wso2.solutions.identity.persistence.dataobject.RelyingPartyDO;
import org.wso2.solutions.identity.persistence.dataobject.VerifierServiceIdpDO;
import org.wso2.solutions.identity.sts.IPPasswordCallback;
import org.wso2.usermanager.custom.jdbc.JDBCRealm;
import org.wso2.usermanager.custom.jdbc.JDBCRealmConfig;
import org.wso2.usermanager.custom.ldap.LDAPRealm;
import org.wso2.usermanager.custom.ldap.LDAPRealmConfig;
import org.wso2.utils.ServerConfiguration;
import org.wso2.utils.ServerException;
import org.wso2.wsas.ServerInitializer;
import org.wso2.wsas.admin.service.UserAdmin;

/**
 * WSO2 WSAS <code>ServerInitializer</code> implementation to carry out
 * initial configuration setup of the Identity Solution : Identity Provider. 
 */
public class Initializer implements ServerInitializer {

    private static Log log = LogFactory.getLog(Initializer.class);

    private static Messages messages = Messages
            .getInstance(IdentityProviderConstants.RESOURCES);

    /**
     * {@inheritDoc}
     * Here we carry out all initialization work of the 
     * identity solution : identity provider
     */
    public void init(ConfigurationContext ctx) throws AxisFault,
            ServerException {
        log.info(messages.getMessage("initStart"));

        ConfigurationContextHolder.setConfigurationContext(ctx);

        try {

            IPPersistenceManager db = IPPersistenceManager
                    .getPersistanceManager();

            ActionDO[] actionDOs = db.getAllActions();
            if (actionDOs.length == 0) {
                //Add actions
                for (int i = 0; i < ActionDO.INITIAL_ACTIONS.length; i++) {
                    ActionDO action = new ActionDO();
                    action.setActionId(ActionDO.INITIAL_ACTIONS[i][0]);
                    action.setActionDescription(ActionDO.INITIAL_ACTIONS[i][1]);
                    db.create(action);
                }
            }

            // Setup the default realm
            RealmConfigurationDO realmConfig = db
                    .getEffectiveRealmConfiguration();
            if (realmConfig == null) {
                if (log.isDebugEnabled()) {
                    log.debug(messages.getMessage("addingDefaultRealminfo"));
                }

                // Add JDBC realm
                RealmDO jdbcRealm = new RealmDO();
                jdbcRealm.setClassName(JDBCRealm.class.getName());
                jdbcRealm.setConfigClassName(JDBCRealmConfig.class.getName());
                db.create(jdbcRealm);

                // Add LDAP realm
                RealmDO ldapRealm = new RealmDO();
                ldapRealm.setClassName(LDAPRealm.class.getName());
                ldapRealm.setConfigClassName(LDAPRealmConfig.class.getName());
                db.create(ldapRealm);

                this.setupSampleRealmConfig(db, jdbcRealm);

                // Add the globally trusted relying party
                RelyingPartyDO rp = new RelyingPartyDO();
                rp = new RelyingPartyDO();
                
                ServerConfiguration serverConfig = ServerConfiguration.getInstance();
                rp.setHostName(serverConfig.getFirstProperty("HostName"));
                rp.setAlias(serverConfig.getFirstProperty("Security.KeyStore.KeyAlias"));
                db.create(rp);

                STSConfigAdmin.configuraServices();
            }

            //Add initial claims
            DialectDO[] dialectDOs = db.getAllSupportedDialects();
            if (dialectDOs.length == 0) {

                if (log.isDebugEnabled()) {
                    log.debug(messages.getMessage("addingInitialClaimsInfo"));
                }

                InitialClaimsProcessor claimsProcessor = new InitialClaimsProcessor();
                claimsProcessor
                        .process(IdentityProviderConstants.INITIAL_CLAIMS_FILE_PATH);
                ArrayList dialects = claimsProcessor.getDialectList();
                for (Iterator iter = dialects.iterator(); iter.hasNext();) {
                    DialectDO dialect = (DialectDO) iter.next();
                    db.create(dialect);
                }

                ArrayList claims = claimsProcessor.getClaimList();
                for (Iterator iter = claims.iterator(); iter.hasNext();) {
                    ClaimDO claim = (ClaimDO) iter.next();
                    if (claim.getUri().equals(IdentityConstants.CLAIM_PPID)) {
                        claim.setUserEditable(false);
                    }
                    db.create(claim);
                }

                // Setup claims for the sample
                ClaimsAdmin claimsAdmin = new ClaimsAdmin();
                ClaimDO[] claimDOs = claimsAdmin.getAllClaims();
                // Set supported
                for (int i = 0; i < claimDOs.length; i++) {

                    if (log.isDebugEnabled()) {
                        log.debug(messages.getMessage("enableClaimsForSample"));
                    }

                    ClaimDO claim = claimDOs[i];
                    if (claim.getUri().equals(
                            IdentityConstants.CLAIM_EMAIL_ADDRESS)) {
                        claim
                                .setAttrId(IdentityProviderConstants.Sample.COLUMN_NAME_EMAIL);
                    } else if (claim.getUri().equals(
                            IdentityConstants.CLAIM_GIVEN_NAME)) {
                        claim
                                .setAttrId(IdentityProviderConstants.Sample.COLUMN_NAME_FIRSTNAME);
                    } else if (claim.getUri().equals(
                            IdentityConstants.CLAIM_SURNAME)) {
                        claim
                                .setAttrId(IdentityProviderConstants.Sample.COLUMN_NAME_LASTNAME);
                    }

                    if (claim.getUri().equals(
                            IdentityConstants.CLAIM_EMAIL_ADDRESS)
                            || claim.getUri().equals(
                                    IdentityConstants.CLAIM_GIVEN_NAME)
                            || claim.getUri().equals(
                                    IdentityConstants.CLAIM_SURNAME)
                            || claim.getUri().equals(
                                    IdentityConstants.CLAIM_PPID)) {
                        claim.setSupported(true);
                        claimsAdmin.updateClaim(claim);
                    }
                }
            }
            
            VerifierServiceIdpDO[] idps = db.getAllVerifierServiceIdpDO();
            if(idps.length==0){
                VerifierServiceIdpDO idp = new VerifierServiceIdpDO();
                idp.setHostName("localhost");
                idp.setGlobal(true);
                db.create(idp);
            }

            setupRelyingPartyUserAccounts();

        } catch (Exception e) {
            throw new AxisFault(e.getMessage(), e);
        }

        // Set the callback handler
        AxisConfiguration axisConfig = ctx.getAxisConfiguration();
        AxisService sts = axisConfig
                .getService(IdentityProviderConstants.SERVICE_NAME_STS_UT);
        Parameter cbHandlerParam = sts
                .getParameter(WSHandlerConstants.PW_CALLBACK_REF);
        if (cbHandlerParam != null) {
            sts.removeParameter(cbHandlerParam);
            if (log.isDebugEnabled()) {
                log.debug(messages.getMessage("removedParameter", new String[] {
                        cbHandlerParam.getName(), sts.getName() }));
            }
        }

        IPPasswordCallback cb = new IPPasswordCallback();
        Parameter param = new Parameter();
        param.setName(WSHandlerConstants.PW_CALLBACK_REF);
        param.setValue(cb);
        sts.addParameter(param);

        if (log.isDebugEnabled()) {
            log.debug(messages.getMessage("addedParameter", new String[] {
                    param.getName(), sts.getName() }));
        }

        log.info(messages.getMessage("initDone"));

    }
    
    /**
     * Setup the user store for the identity provider sample.
     * @param db
     * @param jdbcRealm
     * @throws Exception
     */
    private void setupSampleRealmConfig(IPPersistenceManager db,
            RealmDO jdbcRealm) throws Exception {

        // Add sample realm configuration
        RealmConfigurationDO realmConfig = new RealmConfigurationDO();
        realmConfig.setName("sampleRealm");
        realmConfig.setRealm(jdbcRealm);
        realmConfig.setEffective(true);
        db.create(realmConfig);

        RealmConfigurationPropertyDO prop = new RealmConfigurationPropertyDO();
        prop.setConfig(realmConfig);
        prop.setName("ConnectionURL");
        prop.setValue(IdentityProviderConstants.Sample.CONNECTION_URL);
        db.create(prop);

        prop = new RealmConfigurationPropertyDO();
        prop.setConfig(realmConfig);
        prop.setName("ConnectionUserName");
        prop.setValue(IdentityProviderConstants.Sample.CONNECTION_USER_NAME);
        db.create(prop);

        prop = new RealmConfigurationPropertyDO();
        prop.setConfig(realmConfig);
        prop.setName("ConnectionPassword");
        prop.setValue(IdentityProviderConstants.Sample.CONNECTION_PASSWORD);
        db.create(prop);

        prop = new RealmConfigurationPropertyDO();
        prop.setConfig(realmConfig);
        prop.setName("DriverName");
        prop.setValue(IdentityProviderConstants.Sample.DRIVER);
        db.create(prop);

        prop = new RealmConfigurationPropertyDO();
        prop.setConfig(realmConfig);
        prop.setName("UserTable");
        prop.setValue(IdentityProviderConstants.Sample.USER_TABLE);
        db.create(prop);

        prop = new RealmConfigurationPropertyDO();
        prop.setConfig(realmConfig);
        prop.setName("UserNameColumn");
        prop.setValue(IdentityProviderConstants.Sample.USER_NAME_COLUMN);
        db.create(prop);

        prop = new RealmConfigurationPropertyDO();
        prop.setConfig(realmConfig);
        prop.setName("UserNameColumn");
        prop.setValue(IdentityProviderConstants.Sample.USER_NAME_COLUMN);
        db.create(prop);

        prop = new RealmConfigurationPropertyDO();
        prop.setConfig(realmConfig);
        prop.setName("UserCredentialColumn");
        prop.setValue(IdentityProviderConstants.Sample.PASSWORD_COLUMN);
        db.create(prop);

        prop = new RealmConfigurationPropertyDO();
        prop.setConfig(realmConfig);
        prop.setName("ColumnNames");
        prop.setValue(IdentityProviderConstants.Sample.COLUMN_NAME_EMAIL + ","
                + IdentityProviderConstants.Sample.COLUMN_NAME_FIRSTNAME + ","
                + IdentityProviderConstants.Sample.COLUMN_NAME_LASTNAME);
        db.create(prop);

    }

    private void setupRelyingPartyUserAccounts() throws Exception {
        UserAdmin admin = new UserAdmin();
        String[] roleNames = admin.getRoleNames();
        boolean found = false;
        for (int i = 0; i < roleNames.length; i++) {
            if (IdentityProviderConstants.RP_USER_ROLE.equals(roleNames[i])) {
                found = true;
                break;
            }
        }
        if (!found) {
            admin.addRole(IdentityProviderConstants.RP_USER_ROLE,
                    "Relying Party Users");
        }

    }
}
