/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.admin;

import org.wso2.solutions.identity.IdentityProviderException;
import org.wso2.utils.ServerConfiguration;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.security.KeyStore;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

public class KeystoreUtilAdmin {

    public String importCert(File certificateFile)
            throws IdentityProviderException {
        ServerConfiguration serverConfig = ServerConfiguration.getInstance();
        String storeFilePath = serverConfig
                .getFirstProperty("Security.KeyStore.Location");
        String type = serverConfig.getFirstProperty("Security.KeyStore.Type");
        String passwd = serverConfig
                .getFirstProperty("Security.KeyStore.Password");
        return importCert(certificateFile, storeFilePath, passwd, type);

    }

    public String importCert(File certificateFile, String storeFilePath,
            String passwd, String storeType) throws IdentityProviderException {
        try {

            FileInputStream is = new FileInputStream(storeFilePath);

            KeyStore store = KeyStore.getInstance(storeType);

            store.load(is, passwd.toCharArray());

            CertificateFactory factory = CertificateFactory
                    .getInstance("X.509");
            X509Certificate cert = (X509Certificate) factory
                    .generateCertificate(new FileInputStream(certificateFile));

            // Alias should be the host name
            String name = cert.getSubjectDN().getName();
            String[] parts = name.split(",");
            String alias = null;
            for (int i = 0; i < parts.length; i++) {
                String cnStr = parts[i].trim();
                if (cnStr.startsWith("CN")) {
                    alias = cnStr.substring(3);
                }
            }

            if (alias == null) {
                throw new IdentityProviderException("certNameInvalid",
                        new String[] { name });
            }

            store.setCertificateEntry(alias, cert);

            FileOutputStream os = new FileOutputStream(storeFilePath);
            store.store(os, passwd.toCharArray());
            os.flush();

            return alias;
        } catch (Exception e) {
            throw new IdentityProviderException("errorInCertImport", e);
        }
    }

}
