/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.admin;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.solutions.identity.IdentityProviderConstants;
import org.wso2.solutions.identity.IdentityProviderException;
import org.wso2.solutions.identity.i18n.Messages;
import org.wso2.wsas.admin.service.SecurityScenarioConfigAdmin;
import org.wso2.wsas.persistence.PersistenceManager;
import org.wso2.wsas.persistence.dataobject.KeyStoreDO;

/**
 * This will allow setting the SecurityTokenService security configuration
 */
public class STSConfigAdmin {

    private final static Log log = LogFactory.getLog(STSConfigAdmin.class);

    private final static Messages messages = Messages
            .getInstance(IdentityProviderConstants.RESOURCES);

    public static void configuraServices() throws IdentityProviderException {
        SecurityScenarioConfigAdmin admin = new SecurityScenarioConfigAdmin();
        ConfigurationContext configContext = ConfigurationContextHolder
                .getConfigurationContext();
        admin.setConfigContext(configContext);
        admin.setAxisConfig(configContext.getAxisConfiguration());
        try {
            admin.assignUsersAndRoles(
                    IdentityProviderConstants.SERVICE_NAME_STS_UT, null,
                    "scenario1", new String[] { "admin" }, new String[] {});

            PersistenceManager db = new PersistenceManager();
            KeyStoreDO[] keyStoreDOs = db.getKeyStores();

            // Assuming there's only one keystore
            // Keystore path comparison to find the default keystore doesn't
            // work
            // in windows :-(
            String ksName = keyStoreDOs[0].getKeyStoreName();

            admin.assignUsersAndRolesAndKeyStores(
                    IdentityProviderConstants.SERVICE_NAME_STS_IC, null,
                    "scenario2", new String[] { ksName }, ksName,
                    new String[] { "admin" }, new String[] {});

            admin.assignUsersAndRolesAndKeyStores(
                    IdentityProviderConstants.SERVICE_NAME_STS_IC_SYMM, null,
                    "scenario2", new String[] { ksName }, ksName,
                    new String[] { "admin" }, new String[] {});
        } catch (AxisFault e) {
            log.error(messages
                    .getMessage("errorInChangingSecurityConfiguration"), e);
            throw new IdentityProviderException(
                    "errorInChangingSecurityConfiguration", e);
        }
    }

    // TODO Add the second security scenario

}