/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <string.h>
#include <stdio.h>
#include <axis2_client.h>
#include <axiom_document.h>
#include <axiom_element.h>
#include <axiom_node.h>
#include <axiom_stax_builder.h>
#include <axiom_xml_reader.h>
#include <axutil_utils.h>
#include <axutil_env.h>
#include <oxs_ctx.h>
#include <oxs_utility.h>
#include <oxs_key.h>
#include <oxs_asym_ctx.h>
#include <oxs_axiom.h>
#include <oxs_xml_encryption.h>
#include <oxs_token_encryption_method.h>
#include "httpd.h"
#include "cspace_tvs.h"
#include "process_request.h"
#include "mod_cspace.h"

#define N_CSPACE_OVERRIDE

#define XML_DSIG_NS "http://www.w3.org/2000/09/xmldsig#"
#define XML_ENC_NS "http://www.w3.org/2001/04/xmlenc#"
#define ENC_KEY  "EncryptedKey"
#define ENC_MTD  "EncryptionMethod"
#define KEY_INFO "KeyInfo"

#define find_child(node, name, uri, pfx) \
            (node) ? oxs_axiom_get_first_child_node_by_name \
                        (ctx->env, (node), (name), (uri), (pfx)) : NULL

#define AXIS2C_HOME "AXIS2C_HOME"
#define WSFC_HOME "WSFC_HOME"

#define TEST_RESPONSE \
    "<rp:VerifierResponse xmlns:rp=\"http://ns.wso2.org/relyingparty\" " \
                          "State=\"success\">" \
        "<rp:PropertyBundle>" \
            "<rp:Property Name=\"ldapid\">dumindu</rp:Property>" \
            "<rp:Property Name=\"givenname\">Dumindu</rp:Property>" \
            "<rp:Property Name=\"surname\">Pallewela</rp:Property>" \
            "<rp:Property Name=\"privatepersonalidentifier\">" \
                "qNSahaxByg8A5j+cxqaElz/M1XJ9NpWL/czRDGTOSj8=" \
            "</rp:Property>" \
            "<rp:Property Name=\"emailaddress\">dumindu@wso2.com</rp:Property>" \
        "</rp:PropertyBundle>" \
        "<rp:IssuerInfo>" \
            "<KeyInfo>" \
                "<KeyValue>" \
                    "<RSAKeyValue>" \
                        "<Modulus>" \
                            "vtkhk4/XZrTMPHplQ5x3RxQezGDlCzhxkz+1O2JrzUErMbLl" \
                            "siV0/DGRyFAzL8IdmknpUV9Ri4xxEkYg6OedWThBvoQDvBUR" \
                            "rBwj91ROtvHq2X1UsSJ4smev52AcWOO2iDTishnKFOZk0RlS" \
                            "qgReZIUdRTviQqkNhemvVpwFwBPSEAVFVonF0wx2dMzIUStw" \
                            "s8XnQH615dTj8JSvhlhFznIGy0jvMspGxKGhCSaqWehsJiU7" \
                            "VQHTqj16GRqrmMRWb+29Yo1Bvx8yitEKkjY8n2YzBYPMPDvL" \
                            "XRNn3NKB86216PoDp4mEeWwAZ9xNsc8CpAbXH7K6kyUwGEI/" \
                            "Bg06XQ==" \
                        "</Modulus>" \
                        "<Exponent>AQAB</Exponent>" \
                    "</RSAKeyValue>" \
                "</KeyValue>" \
            "</KeyInfo>" \
        "</rp:IssuerInfo>" \
    "</rp:VerifierResponse>"

static void * AXIS2_CALL process_ctx_malloc(axutil_allocator_t *alloc, size_t sz)
{
    return apr_palloc((apr_pool_t *)(alloc->current_pool), sz);
}

/*NOTE: Not impld, we don't need this*/
static void * AXIS2_CALL process_ctx_realloc(axutil_allocator_t *alloc, void *ptr,
                                      size_t sz)
{
    return NULL;
}

/*handled by apr pools*/
static void AXIS2_CALL process_ctx_free(axutil_allocator_t *alloc, void *ptr)
{
    return;
}

process_ctx_t* cspace_process_ctx_create(void *pool)
{
    /* TODO: we need ctx, alloc and the env to be allocated using the global pool
     * r->server->process->pool?? 
     */

    process_ctx_t *ctx;
    ctx = (process_ctx_t *)apr_palloc((apr_pool_t*)pool, sizeof(process_ctx_t));
    if (!ctx)
        return NULL;
    ctx->allocator =
        (axutil_allocator_t *)apr_palloc((apr_pool_t*)pool, 
                                         sizeof(axutil_allocator_t));
    if (!ctx->allocator)
        return NULL;

    ctx->pool = pool;

    ctx->allocator->malloc_fn = process_ctx_malloc;
    ctx->allocator->realloc = process_ctx_realloc;
    ctx->allocator->free_fn = process_ctx_free;
    ctx->allocator->local_pool = pool;
    ctx->allocator->global_pool = pool;
    ctx->allocator->current_pool = pool;

    ctx->env = axutil_env_create(ctx->allocator);
    if (!ctx->env) {
        AXIS2_FREE(ctx->allocator, ctx->allocator);
        ctx->allocator = NULL;
        return NULL;
    }
    
    return ctx;
}

static axiom_node_t* make_om_node(process_ctx_t *ctx, const axis2_char_t* req)
{
    /*TODO: find the objs that need not be in ctx and remove them from ctx*/
    printf("Node: %s\n", req);
    ctx->reader =
        axiom_xml_reader_create_for_memory(ctx->env, (void *)req,
                                           (int)strlen(req), "UTF-8",
                                           AXIS2_XML_PARSER_TYPE_BUFFER);
    if (!ctx->reader)
        return NULL;

    if (!(ctx->builder = axiom_stax_builder_create(ctx->env, ctx->reader)))
        return NULL;

    if (!(ctx->doc = axiom_document_create(ctx->env, NULL, ctx->builder)))
        return NULL;

    if (!(ctx->node = axiom_document_build_all(ctx->doc, ctx->env)))
        return NULL;

    return ctx->node;
}

static int get_session_key(process_ctx_t *ctx, axiom_node_t *node, 
                           oxs_key_t **sess_key)
{
    axiom_node_t *enc_key_node = NULL;
    axiom_node_t *enc_mtd_node = NULL;
    oxs_asym_ctx_t *asym_ctx = NULL;
    axis2_char_t *enc_algo = NULL;
    int rv;
    
    axiom_node_t *tempnode = find_child(node, KEY_INFO, XML_DSIG_NS, NULL);

    enc_key_node = find_child(tempnode, ENC_KEY, XML_ENC_NS, "e");
    if (!enc_key_node) return FAIL;

    enc_mtd_node = find_child(enc_key_node, ENC_MTD, XML_ENC_NS, "e");
    if (!enc_mtd_node) return FAIL;
    
    enc_algo = oxs_token_get_encryption_method(ctx->env, enc_mtd_node);
    if (!enc_algo) return FAIL;
    
    asym_ctx = oxs_asym_ctx_create(ctx->env);
    if (!asym_ctx) return FAIL;

    oxs_asym_ctx_set_file_name(asym_ctx, ctx->env, ctx->key_file);
    oxs_asym_ctx_set_format(asym_ctx, ctx->env,
                            oxs_util_get_format_by_file_extension(ctx->env,
                                                                  ctx->key_file
                                                                  ));
    oxs_asym_ctx_set_operation(asym_ctx, ctx->env, 
                               OXS_ASYM_CTX_OPERATION_PRV_DECRYPT); 
    /*oxs_asym_ctx_set_password(asym_ctx, ctx->env, "1234");*/ /*TODO:MAGIC*/
    oxs_asym_ctx_set_algorithm(asym_ctx, ctx->env, enc_algo);

    *sess_key = oxs_key_create(ctx->env);
    if (!(*sess_key))
        return FAIL;

    rv = oxs_xml_enc_decrypt_key(ctx->env, asym_ctx, enc_key_node,
                                 enc_key_node, *sess_key);

    return rv;
}

static int get_decrypted_node(process_ctx_t *ctx, axiom_node_t *node,
                              oxs_key_t *key, axiom_node_t **dec_node)
{
    oxs_ctx_t *sym_ctx = NULL;
    int rv;   

    sym_ctx = oxs_ctx_create(ctx->env);
    if (!sym_ctx) return FAIL;

    oxs_ctx_set_key(sym_ctx, ctx->env, key);
    rv = oxs_xml_enc_decrypt_node(ctx->env, sym_ctx, node, dec_node);
    
    return rv;
}

static axiom_node_t* decrypt_node(process_ctx_t *ctx, axiom_node_t *node)
{
    oxs_key_t *sess_key = NULL;
    axiom_node_t *dec_node = NULL;

    if (!get_session_key(ctx, node, &sess_key))
        return NULL;

    if (!get_decrypted_node(ctx, node, sess_key, &dec_node))
        return NULL;

#ifdef CSPACE_DEBUG
    printf("\ndec:\n%s\n\n",axiom_node_to_string(dec_node, ctx->env));
#endif

    return dec_node;
}

int init_axis2_client(process_ctx_t *ctx, char *tvs)
{
    char* client_home = NULL;

    ctx->tvs = axis2_endpoint_ref_create(ctx->env, tvs);
    if (!ctx->tvs)
        return FAIL;
    
    ctx->options = axis2_options_create(ctx->env);
    if (!ctx->options)
        return FAIL;
    
    axis2_options_set_to(ctx->options, ctx->env, ctx->tvs);
    axis2_options_set_action(ctx->options, ctx->env, CSPACE_TVS_ACTION_VERIFY);

    client_home = AXIS2_GETENV(WSFC_HOME);
    if (!client_home) {
        client_home = AXIS2_GETENV(AXIS2C_HOME);
        if (!client_home) {
#ifdef CSPACE_DEBUG
            printf("ERROR: Client repo not set\n");
#endif
            return FAIL;
        }
    }

#ifdef CSPACE_DEBUG
    printf("Using Client repo:%s\n", client_home);
#endif
    
    ctx->svc_client = axis2_svc_client_create(ctx->env, client_home);
    if (!ctx->svc_client)
        return FAIL;

    axis2_svc_client_set_options(ctx->svc_client, ctx->env, ctx->options);

    axis2_svc_client_engage_module(ctx->svc_client, ctx->env,
                                   AXIS2_MODULE_ADDRESSING);

    return SUCC;
}

static int read_issuer_info(process_ctx_t *ctx, axiom_node_t *prop_bndl_node,
                            axiom_element_t *prop_bndl_ele, void *ret_hash,
                            hash_set_cb_t cb)
{
    return SUCC;
}

static int read_all_properties(process_ctx_t *ctx,
                               axiom_node_t *prop_bndl_node,
                               axiom_element_t *prop_bndl_ele,
                               void *ret_hash, hash_set_cb_t cb)
{
    axiom_child_element_iterator_t *prop_itr = NULL;
    axiom_node_t *prop_node = NULL;
    axiom_element_t *prop_ele = NULL;
    char *attr_val;
    char *text_val;
    char *key_val;

    prop_itr = axiom_element_get_child_elements(prop_bndl_ele, ctx->env,
                                                prop_bndl_node);
    if (prop_itr) {
        while (AXIOM_CHILD_ELEMENT_ITERATOR_HAS_NEXT(prop_itr, ctx->env)) {
            prop_node = AXIOM_CHILD_ELEMENT_ITERATOR_NEXT(prop_itr, ctx->env);
            if (prop_node) {
                prop_ele = axiom_node_get_data_element(prop_node, ctx->env);
                if (prop_ele) {
                    attr_val = axiom_element_get_attribute_value_by_name(
                            prop_ele, ctx->env, CSPACE_TVS_RES_PROPERTY_ATTR);

                    text_val = axiom_element_get_text(prop_ele, ctx->env,
                                                      prop_node);

                    key_val = AXIS2_MALLOC(ctx->allocator,
                                           strlen(attr_val) + 11);
                                           /*10 == strlen("CARDSPACE_") + 1*/

                    if (!key_val) {
                        return FAIL;
                        /*mem overflow*/
                        /*TODO:log this*/
                    }
    

                    if (attr_val && text_val) {
                        strcpy(key_val, CSPACE_HEADER_PREFIX);
                        strcat(key_val, attr_val);

                        /*we can't use apr_pstrcat here as this is used by IIS*/

                        /*key_val = (char *)
                         *    apr_pstrcat((apr_pool_t *)(ctx->pool),
                                          "cardspace_", attr_val, NULL);*/
                        (*cb)(key_val, text_val, ret_hash, ctx->pool);
                    } else {
                        /*TODO:logerror;*/
                    }

                }
            }
        }
    }

    return SUCC;
}

static int process_response(process_ctx_t *ctx, axiom_node_t *node,
                            void* ret_hash, hash_set_cb_t cb)
{
    axiom_element_t *vr_ele;
    axiom_node_t *n = NULL;
    axiom_element_t *e = NULL;
    char *attr_val = NULL;

#ifdef CSPACE_DEBUG
    printf(axiom_node_to_string(node, ctx->env));
#endif

    vr_ele = axiom_node_get_data_element(node, ctx->env);
    if (!vr_ele)
        return FAIL; /*Response not understood*/
    if (!(verify_element_name(ctx, vr_ele, CSPACE_TVS_RES_ROOT)))
        return FAIL; /*Response not understood*/
    
    attr_val = 
        axiom_element_get_attribute_value_by_name(vr_ele, ctx->env,
                                                  CSPACE_TVS_RES_STATE_ATTR);
    
    /* This logic seems faulty. We do not have to return the verification
     * state using return value. It is set using the callback cb.
     */
    /*if (strcmp(attr_val, CSPACE_TVS_RES_STATE_AUTHED) == 0) {
        state = CSPACE_STATE_AUTHED;
    } else if (strcmp(attr_val, CSPACE_TVS_RES_STATE_PENDING) == 0) {
        state = CSPACE_STATE_PENDING;
    }*/
    
    /*no dup no free for attr_val... read the comment inside callback fn*/
    (*cb)(CSPACE_TVS_RES_STATE_HEADER, attr_val, ret_hash, ctx->pool);
    
    n = axiom_node_get_first_element(node, ctx->env);
    if (!n)
        return FAIL; /*No content*/

    while (n) {
        e = axiom_node_get_data_element(n, ctx->env);

        if (verify_element_name(ctx, e, CSPACE_TVS_RES_PROPERTY_BUNDLE)) {
            if (!read_all_properties(ctx, n, e, ret_hash, cb))
                return FAIL;
        } else if (verify_element_name(ctx, e, CSPACE_TVS_RES_ISSUER_INFO)) {
            if (!read_issuer_info(ctx, n, e, ret_hash, cb))
                return FAIL;
        } else {
            /*Invalid*/
            return FAIL;

            /* even though this element invalid, should we return FAIL here?
             * what should be the behaviour if the verifier response is not in
             * the exact format we expect?? */
        }
  
        n = axiom_node_get_next_sibling(n, ctx->env);
    }

    return SUCC;
}

static axiom_node_t* create_tvs_node(process_ctx_t *ctx, 
                                     axiom_node_t *saml_node,
                                     const char *issuer)
{
    axiom_node_t *vr_node = NULL;
    
    axiom_node_t *node, *txt_node;
    axiom_element_t *ele, *vr_ele;
    axiom_namespace_t *ns;
    axiom_text_t *issuer_txt;

    ns = axiom_namespace_create(ctx->env, CSPACE_RP_NAMESPACE, "rp");
    vr_ele = axiom_element_create(ctx->env, NULL, CSPACE_TVS_REQ_ROOT, ns,
                                  &vr_node);
    ele = axiom_element_create(ctx->env, vr_node, CSPACE_TVS_REQ_TOKEN, ns,
                               &node);
    axiom_node_add_child(node, ctx->env, saml_node);
    ele = axiom_element_create(ctx->env, vr_node, CSPACE_TVS_REQ_ISSUER, ns,
                               &node);
    issuer_txt = axiom_text_create(ctx->env, node, issuer, &txt_node);
    return vr_node;
}

int cspace_process_request(process_ctx_t *ctx, char *req, char* key_file,
                           char* tvs, void *ret_hash, hash_set_cb_t cb)
{
    axiom_node_t *root_node, *ret_node;
    axiom_node_t *dec_saml_node;
    int state = FAIL;

    ctx->key_file = key_file; /*no need to strcpy, read only*/

    req = strstr((const char *)req, CSPACE_XML_TOKEN) + /*xmlToken=*/
          strlen(CSPACE_XML_TOKEN);

    root_node = make_om_node(ctx, req);
    if (!(root_node))
        return FAIL;

#ifdef CSPACE_DEBUG
    printf("\naxiom serialized request:\n %s \n\n",
           axiom_node_to_string(root_node, ctx->env));
#endif

    dec_saml_node = decrypt_node(ctx, root_node);
    if (!dec_saml_node)
        return FAIL;
    
    if (!init_axis2_client(ctx, tvs))
        return FAIL;

#ifdef CSPACE_OVERRIDE
    ret_node = make_om_node(ctx, TEST_RESPONSE);
    if (!ret_node) {
        printf("\n\nTEST NODE NOT CREATED!!!!!\n");
        return FAIL;
    }
#else

    dec_saml_node = create_tvs_node(ctx, dec_saml_node, NULL);
    
    ret_node = invoke_tvs(ctx, dec_saml_node);
    
    if (!ret_node) {
        printf("\n\nTVS INVOKE FAILED!!!!!\n");
        return FAIL;
    }
#endif
    
#ifdef CSPACE_DEBUG
    printf("received node: %s\n\n", axiom_node_to_string(ret_node, ctx->env));
#endif

    state = process_response(ctx, ret_node, ret_hash, cb);
   
    /*process_ctx_free(ctx); no free, using apr pools */

    return state;
}
