/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.relyingparty;

import java.util.ArrayList;

/**
 * This holds all relying party information required by the token verifier in
 * verifying tokens
 */
public class RelyingPartyData {

    /**
     * Username of the relying party. The relying party is registered with the
     * TokenVerifier service with this username.
     */
    private String userName;

    /**
     * Password used along with the username of the relying party to
     * authenticate with the relying party service.
     */
    private String password;

    /**
     * The list of X509Certificates of the trusted identity providers.
     */
    private ArrayList trusteIdentityProviderCertificates;

    /**
     * This indicates whether the relying party allows self issued tokens.
     */
    private boolean allowSelfIssued = true;

    /**
     * This indicates whether the relying party wants to turn off verification
     * of the certificate used to sign the token.
     */
    private boolean disableTrustVerification;

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public ArrayList getTrusteIdentityProviderCertificates() {
        return trusteIdentityProviderCertificates;
    }

    public void setTrusteIdentityProviderCertificates(
            ArrayList trusteIdentityProviderCertificates) {
        this.trusteIdentityProviderCertificates = trusteIdentityProviderCertificates;
    }

    public boolean isAllowSelfIssued() {
        return allowSelfIssued;
    }

    public void setAllowSelfIssued(boolean allowSelfIssued) {
        this.allowSelfIssued = allowSelfIssued;
    }

    public boolean isDisableTrustVerification() {
        return disableTrustVerification;
    }

    public void setDisableTrustVerification(boolean noTrustVerification) {
        this.disableTrustVerification = noTrustVerification;
    }

}
