/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.relyingparty.servletfilter;

import org.apache.axiom.om.OMAttribute;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.description.TransportOutDescription;
import org.apache.axis2.util.XMLUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.security.WSSConfig;
import org.w3c.dom.Element;
import org.wso2.solutions.identity.relyingparty.TokenVerifier;
import org.wso2.solutions.identity.relyingparty.TokenVerifierConstants;
import org.wso2.solutions.identity.relyingparty.util.CustomizedSSLTransport;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.xml.namespace.QName;

import java.io.FileInputStream;
import java.io.IOException;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.util.Iterator;

public class RelyingPartyFilter implements Filter {

    public static Log log = LogFactory.getLog(RelyingPartyFilter.class);

    static {
        WSSConfig.getDefaultWSConfig();
    }

    FilterConfig filterConfig;

    public void doFilter(ServletRequest request, ServletResponse response,
            FilterChain chain) throws IOException, ServletException {
        try {

            String path = ((HttpServletRequest) request).getRequestURI();

            if (log.isDebugEnabled()) {
                log.debug("RequestURI : " + path);
            }

            String configLocationParam = filterConfig
                    .getInitParameter("config-file");

            RelyingPartyFilterConfiguration config = RelyingPartyFilterConfiguration
                    .getInstance(configLocationParam);

            String infoCardSignin = request.getParameter("InfoCardSignin");

            if (log.isDebugEnabled()) {
                log.debug("InfoCardSignin : " + infoCardSignin);
            }

            if (infoCardSignin != null && "Log in".equals(infoCardSignin)) {

                if (log.isDebugEnabled()) {
                    log.debug("InfoCardSignin=Log in");
                }

                String xmlToken = request.getParameter("xmlToken");
                if (xmlToken != null) {

                    PrivateKey privKey = getPrivateKey(config);
                    TokenVerifier verifier = new TokenVerifier();
                    Element plainTokenElem = verifier.decryptToken(xmlToken,
                            privKey);
                    OMElement verificationResponseElem = verifyToken(
                            plainTokenElem, config);

                    OMAttribute attr = verificationResponseElem
                            .getAttribute(new QName(
                                    TokenVerifierConstants.ATTR_STATE));
                    request.setAttribute(TokenVerifierConstants.SERVLET_ATTR_STATE,
                            attr.getAttributeValue());

                    request
                            .setAttribute(
                                    TokenVerifierConstants.ISSUER_INFO,
                                    verificationResponseElem
                                            .getFirstChildWithName(
                                                    new QName(
                                                            TokenVerifierConstants.NS,
                                                            TokenVerifierConstants.LN_ISSUER_INFO))
                                            .getFirstElement().toString());

                    OMElement prpertiesElem = verificationResponseElem
                            .getFirstChildWithName(new QName(
                                    TokenVerifierConstants.NS,
                                    TokenVerifierConstants.LN_PROPERTIES));
                    Iterator propertyElems = prpertiesElem
                            .getChildrenWithName(new QName(
                                    TokenVerifierConstants.NS,
                                    TokenVerifierConstants.LN_PROPERTY,
                                    TokenVerifierConstants.PREFIX));
                    while (propertyElems.hasNext()) {
                        OMElement property = (OMElement) propertyElems.next();
                        OMAttribute nameAttr = property.getAttribute(new QName(
                                TokenVerifierConstants.ATTR_NAME));
                        request.setAttribute(nameAttr.getAttributeValue(),
                                property.getText());
                    }

                } else {
                    request.setAttribute(TokenVerifierConstants.ATTR_STATE, TokenVerifierConstants.STATE_FAILURE);
                    request.setAttribute(TokenVerifierConstants.FAILURE_REASON, TokenVerifierConstants.REASON_TOKEN_MISSING);
                }
            }
        } catch (Exception e) {
            request.setAttribute(TokenVerifierConstants.ATTR_STATE, TokenVerifierConstants.STATE_FAILURE);
            log.error("Error in token verification", e);
        }
        chain.doFilter(request, response);
    }

    /**
     * Access the verifier service and obtain the verification response
     * 
     * @param plainTokenElem
     * @param config
     * @return
     */
    protected OMElement verifyToken(Element plainTokenElem,
            RelyingPartyFilterConfiguration config) throws Exception {
        OMElement elem = XMLUtils.toOM(plainTokenElem);
        OMFactory fac = elem.getOMFactory();

        OMElement reqElem = fac.createOMElement(new QName(
                TokenVerifierConstants.NS, TokenVerifierConstants.LN_REQ,
                TokenVerifierConstants.PREFIX));
        OMElement tokenElem = fac.createOMElement(new QName(
                TokenVerifierConstants.NS, TokenVerifierConstants.LN_TOKEN,
                TokenVerifierConstants.PREFIX), reqElem);
        tokenElem.addChild(elem);
        OMElement issuerElem = fac.createOMElement(new QName(
                TokenVerifierConstants.NS, TokenVerifierConstants.LN_ISSUER,
                TokenVerifierConstants.PREFIX), reqElem);
        issuerElem.setText(TokenVerifierConstants.ISSUER_SELF);

        ServiceClient client = new ServiceClient(ConfigurationContextFactory
                .createDefaultConfigurationContext(), null);

        Options options = new Options();
        options.setAction("urn:verify");
        options.setTo(new EndpointReference(config.getVerifierURL()));

        if (config.getDoCustomizeSSL()) {
            CustomizedSSLTransport sender = new CustomizedSSLTransport();
            sender.setTrustStoreFileName(config.getTrustStoreFileName());
            sender.setTrustStorePass(config.getTrustStorePass());
            TransportOutDescription description = new TransportOutDescription(
                    "https");
            description.setSender(sender);
            options.setTransportOut(description);
        }
        client.setOptions(options);
        OMElement responseElement = client.sendReceive(reqElem);
        log.debug(responseElement);
        return responseElement;

    }

    private PrivateKey getPrivateKey(RelyingPartyFilterConfiguration config)
            throws Exception {
        KeyStore store = KeyStore.getInstance(config.getServiceStoreType());

        store.load(new FileInputStream(config.getServiceKeystore()), config
                .getServiceStorePass().toCharArray());

        return (PrivateKey) store.getKey(config.getServiceKeyAlias(), config
                .getServiceKeyPass().toCharArray());
    }

    public void destroy() {
    }

    public void init(FilterConfig config) throws ServletException {
        this.filterConfig = config;
    }

}