package org.wso2.solutions.identity.relyingparty.util;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.OperationContext;
import org.apache.axis2.i18n.Messages;
import org.apache.axis2.transport.MessageFormatter;
import org.apache.axis2.transport.http.AbstractHTTPSender;
import org.apache.axis2.transport.http.AxisRequestEntity;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.axis2.wsdl.WSDLConstants;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpMethodBase;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.protocol.Protocol;
import org.apache.commons.httpclient.protocol.ProtocolSocketFactory;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class CustomizedSSLSender extends AbstractHTTPSender {

    private static final Log log = LogFactory.getLog(CustomizedSSLSender.class);

    private String trustStoreFileName = null;

    private String trustStorePass = null;

    public void send(MessageContext msgContext, URL url, String soapActionString)
            throws IOException {

        HttpClient httpClient = getHttpClient(msgContext);

        /*
         * What's up with this, it never gets used anywhere?? --Glen String
         * charEncoding = (String)
         * msgContext.getProperty(Constants.Configuration.CHARACTER_SET_ENCODING);
         * 
         * if (charEncoding == null) { charEncoding =
         * MessageContext.DEFAULT_CHAR_SET_ENCODING; }
         */

        PostMethod postMethod = new PostMethod();
        MessageFormatter messageFormatter = populateCommonProperties(
                msgContext, url, postMethod, httpClient, soapActionString);

        postMethod.setRequestEntity(new AxisRequestEntity(messageFormatter,
                msgContext, format, soapActionString, chunked, isAllowedRetry));

        if (!httpVersion.equals(HTTPConstants.HEADER_PROTOCOL_10) && chunked) {
            postMethod.setContentChunked(true);
        }

        String soapAction = messageFormatter.formatSOAPAction(msgContext,
                format, soapActionString);

        if (soapAction != null) {
            postMethod.setRequestHeader(HTTPConstants.HEADER_SOAP_ACTION,
                    soapAction);
        }

        URL keystoreURL = null;

        try {
            keystoreURL = new URL("file", "localhost", -1, trustStoreFileName);
        } catch (MalformedURLException e1) {
            e1.printStackTrace();
        }

        ProtocolSocketFactory factory = new AuthSSLProtocolSocketFactory(null,
                null, keystoreURL, trustStorePass);

        //register
        Protocol.registerProtocol("https", new Protocol("https", factory, 443));

        /*
         * main excecution takes place..
         */
        try {
            executeMethod(httpClient, msgContext, url, postMethod);
            handleResponse(msgContext, postMethod);
        } catch (IOException e) {
            log.info("Unable to sendViaPost to url[" + url + "]", e);
            throw AxisFault.makeFault(e);
        } finally {
            cleanup(msgContext, postMethod);
        }

        Protocol.unregisterProtocol("https");

    }

    private void cleanup(MessageContext msgContext, HttpMethod method) {
        if (msgContext.isPropertyTrue(HTTPConstants.AUTO_RELEASE_CONNECTION)) {
            method.releaseConnection();
        }
    }

    /**
     * Used to handle the HTTP Response
     *
     * @param msgContext - The MessageContext of the message
     * @param method     - The HTTP method used
     * @throws IOException - Thrown in case an exception occurs
     */
    private void handleResponse(MessageContext msgContext, HttpMethodBase method)
            throws IOException {

        int statusCode = method.getStatusCode();
        if (statusCode == HttpStatus.SC_OK) {
            processResponse(method, msgContext);
        } else if (statusCode == HttpStatus.SC_ACCEPTED) {
        } else if (statusCode == HttpStatus.SC_INTERNAL_SERVER_ERROR
                || statusCode == HttpStatus.SC_BAD_REQUEST) {
            Header contenttypeHeader = method
                    .getResponseHeader(HTTPConstants.HEADER_CONTENT_TYPE);
            String value = null;
            if (contenttypeHeader != null) {
                value = contenttypeHeader.getValue();
            }
            OperationContext opContext = msgContext.getOperationContext();
            if (opContext != null) {
                MessageContext inMessageContext = opContext
                        .getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
                if (inMessageContext != null) {
                    inMessageContext.setProcessingFault(true);
                }
            }
            if (value != null) {

                processResponse(method, msgContext);
            }
            Object isTransportNonBlocking = msgContext
                    .getProperty(MessageContext.TRANSPORT_NON_BLOCKING);
            if (isTransportNonBlocking != null
                    && ((Boolean) isTransportNonBlocking).booleanValue()) {
                throw new AxisFault(Messages.getMessage("transportError",
                        String.valueOf(statusCode), method.getStatusText()));
            }
        } else {
            throw new AxisFault(Messages.getMessage("transportError", String
                    .valueOf(statusCode), method.getStatusText()));
        }
    }

    public void setTrustStoreFileName(String keyStoreFileName) {
        this.trustStoreFileName = keyStoreFileName;
    }

    public void setTrustStorePass(String keyStorePass) {
        this.trustStorePass = keyStorePass;
    }

}
