package org.wso2.solutions.identity.relyingparty.util;

import org.apache.axiom.om.OMOutputFormat;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.description.TransportOutDescription;
import org.apache.axis2.transport.TransportUtils;
import org.apache.axis2.transport.http.CommonsHTTPTransportSender;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.axis2.transport.http.HTTPTransportUtils;
import org.apache.axis2.util.JavaUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.xml.stream.FactoryConfigurationError;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

public class CustomizedSSLTransport extends CommonsHTTPTransportSender {

    private static final Log log = LogFactory
            .getLog(CustomizedSSLTransport.class);

    private String trustStoreFileName = null;

    private String trustStorePass = null;

    public InvocationResponse invoke(MessageContext msgContext)
            throws AxisFault {
        try {
            OMOutputFormat format = new OMOutputFormat();
            // if (!msgContext.isDoingMTOM())
            msgContext.setDoingMTOM(HTTPTransportUtils.doWriteMTOM(msgContext));
            msgContext.setDoingSwA(HTTPTransportUtils.doWriteSwA(msgContext));
            msgContext.setDoingREST(HTTPTransportUtils.isDoingREST(msgContext));
            format.setSOAP11(msgContext.isSOAP11());
            format.setDoOptimize(msgContext.isDoingMTOM());
            format.setDoingSWA(msgContext.isDoingSwA());
            format.setCharSetEncoding(HTTPTransportUtils
                    .getCharSetEncoding(msgContext));

            Object mimeBoundaryProperty = msgContext
                    .getProperty(Constants.Configuration.MIME_BOUNDARY);
            if (mimeBoundaryProperty != null) {
                format.setMimeBoundary((String) mimeBoundaryProperty);
            }

            TransportOutDescription transportOut = msgContext
                    .getConfigurationContext().getAxisConfiguration()
                    .getTransportOut(Constants.TRANSPORT_HTTP);

            // if a parameter has set been set, we will omit the SOAP action for
            // SOAP 1.2
            if (transportOut != null) {
                if (!msgContext.isSOAP11()) {
                    Parameter param = transportOut
                            .getParameter(HTTPConstants.OMIT_SOAP_12_ACTION);
                    Object parameterValue = null;
                    if (param != null) {
                        parameterValue = param.getValue();
                    }

                    if (parameterValue != null
                            && JavaUtils.isTrueExplicitly(parameterValue)) {
                        // Check whether user has already overridden this.
                        Object propertyValue = msgContext
                                .getProperty(Constants.Configuration.DISABLE_SOAP_ACTION);
                        if (propertyValue == null
                                | !JavaUtils.isFalseExplicitly(propertyValue)) {
                            msgContext
                                    .setProperty(
                                            Constants.Configuration.DISABLE_SOAP_ACTION,
                                            Boolean.TRUE);
                        }
                    }
                }
            }

            // Transport URL can be different from the WSA-To. So processing
            // that now.
            EndpointReference epr = null;
            String transportURL = (String) msgContext
                    .getProperty(Constants.Configuration.TRANSPORT_URL);

            if (transportURL != null) {
                epr = new EndpointReference(transportURL);
            } else if (msgContext.getTo() != null
                    && !msgContext.getTo().hasAnonymousAddress()) {
                epr = msgContext.getTo();
            }

            // Check for the REST behavior, if you desire rest behavior
            // put a <parameter name="doREST" value="true"/> at the
            // server.xml/client.xml file
            // ######################################################
            // Change this place to change the wsa:toepr
            // epr = something
            // ######################################################

            if (epr != null) {
                if (!epr.hasNoneAddress()) {
                    writeMessageWithConfiguredHTTPClient(msgContext, epr,
                            format);
                    TransportUtils.setResponseWritten(msgContext, true);
                }
            }

            /*
             * else { if (msgContext.getProperty(MessageContext.TRANSPORT_OUT) !=
             * null) { sendUsingOutputStream(msgContext, format);
             * TransportUtils.setResponseWritten(msgContext, true); } else {
             * throw new AxisFault( "Both the TO and
             * MessageContext.TRANSPORT_OUT property " + "are null, so nowhere
             * to send"); } }
             */
        } catch (FactoryConfigurationError e) {
            log.debug(e);
            throw AxisFault.makeFault(e);
        } catch (IOException e) {
            log.debug(e);
            throw AxisFault.makeFault(e);
        }
        return InvocationResponse.CONTINUE;
    }

    private void writeMessageWithConfiguredHTTPClient(
            MessageContext messageContext, EndpointReference toEPR,
            OMOutputFormat format) throws AxisFault {

        try {
            URL url = new URL(toEPR.getAddress());

            CustomizedSSLSender sender = new CustomizedSSLSender();
            sender.setTrustStoreFileName(trustStoreFileName);
            sender.setTrustStorePass(trustStorePass);

            // Following order needed to be preserved because,
            // HTTP/1.0 does not support chunk encoding
            sender.setChunked(true);
            sender.setHttpVersion(httpVersion);
            sender.setFormat(format);

            sender.send(messageContext, url, findSOAPAction(messageContext));

        } catch (MalformedURLException e) {
            log.debug(e);
            throw AxisFault.makeFault(e);
        } catch (IOException e) {
            log.debug(e);
            throw AxisFault.makeFault(e);
        }
    }

    private static String findSOAPAction(MessageContext messageContext) {
        String soapActionString = null;

        Parameter parameter = messageContext.getTransportOut().getParameter(
                HTTPConstants.OMIT_SOAP_12_ACTION);
        if (parameter != null
                && JavaUtils.isTrueExplicitly(parameter.getValue())
                && !messageContext.isSOAP11()) {
            return "\"\"";
        }

        Object disableSoapAction = messageContext.getOptions().getProperty(
                Constants.Configuration.DISABLE_SOAP_ACTION);

        if (!JavaUtils.isTrueExplicitly(disableSoapAction)) {
            // first try to get the SOAP action from message context
            soapActionString = messageContext.getSoapAction();
            if ((soapActionString == null) || (soapActionString.length() == 0)) {
                // now let's try to get WSA action
                soapActionString = messageContext.getWSAAction();
                if (messageContext.getAxisOperation() != null
                        && ((soapActionString == null) || (soapActionString
                                .length() == 0))) {
                    // last option is to get it from the axis operation
                    soapActionString = messageContext.getAxisOperation()
                            .getSoapAction();
                }
            }
        }

        // Since action is optional for SOAP 1.2 we can return null here.
        if (soapActionString == null && messageContext.isSOAP11()) {
            soapActionString = "\"\"";
        }

        return soapActionString;
    }

    public String getTrustStoreFileName() {
        return trustStoreFileName;
    }

    public String getTrustStorePass() {
        return trustStorePass;
    }

    public void setTrustStoreFileName(String keyStorefile) {
        this.trustStoreFileName = keyStorefile;
    }

    public void setTrustStorePass(String keyStorePass) {
        this.trustStorePass = keyStorePass;
    }

}
